{ pkgs, ... }:
{
  programs.kitty = {
    enable = true;
    # themeFile = "rose-pine";
    font.name = "CaskaydiaCove Nerd Font";
    shellIntegration.enableFishIntegration = true;
    shellIntegration.enableBashIntegration = true;
    settings = {
      font_size = 12;
      window_padding_width = "8 8";
      confirm_os_window_close = -1;
      enable_audio_bell = "no";
      background_opacity = pkgs.lib.mkForce "0.8";
      allow_remote_control = "yes";
      listen_on = "unix:/tmp/kitty";
      scrollback_pager = ''nvim --noplugin -c "set signcolumn=no showtabline=0" -c "silent write! /tmp/kitty_scrollback_buffer | te cat /tmp/kitty_scrollback_buffer - "'';
      cursor = pkgs.lib.mkForce "#c0caf5";
      cursor_text_color = pkgs.lib.mkForce "#1a1b26";
      cursor_trail = 3;
    };
    keybindings = {
      "kitty_mod+h" = "show_scrollback";
    };
  };

  programs.neovide = {
    enable = false;
    settings = {
      font = {
        normal = [ "CaskaydiaCove Nerd Font" ];
        size = 13;
      };
    };
  };

  xdg.mimeApps = {
    enable = true;
    defaultApplications = {
      "text/html" = [ "zen.desktop" ];
      "x-scheme-handler/http" = [ "zen.desktop" ];
      "x-scheme-handler/https" = [ "zen.desktop" ];
      "x-scheme-handler/about" = [ "zen.desktop" ];
      "x-scheme-handler/unknown" = [ "zen.desktop" ];
    };
  };

  # programs.firefox = {
  #   enable = true;
  #   # package = pkgs.librewolf;
  #   profiles = {
  #     Personal = {
  #       name = "Youwen Wu";
  #       settings = {
  #         webgl.disabled = false;
  #         privacy.resistFingerprinting = false;
  #         privacy.clearOnShutdown.history = false;
  #         privacy.clearOnShutdown.cookies = false;
  #         network.cookie.lifetimePolicy = 0;
  #         search.default = "Google";
  #         search.force = true;
  #         search.engines = {
  #           "Nix Packages" = {
  #             urls = [
  #               {
  #                 template = "https://search.nixos.org/packages";
  #                 params = [
  #                   {
  #                     name = "type";
  #                     value = "packages";
  #                   }
  #                   {
  #                     name = "query";
  #                     value = "{searchTerms}";
  #                   }
  #                 ];
  #               }
  #             ];
  #
  #             icon = "${pkgs.nixos-icons}/share/icons/hicolor/scalable/apps/nix-snowflake.svg";
  #             definedAliases = ["@np"];
  #           };
  #
  #           "NixOS Wiki" = {
  #             urls = [{template = "https://wiki.nixos.org/index.php?search={searchTerms}";}];
  #             iconUpdateURL = "https://wiki.nixos.org/favicon.png";
  #             updateInterval = 24 * 60 * 60 * 1000; # every day
  #             definedAliases = ["@nw"];
  #           };
  #
  #           "Google" = {
  #             urls = [
  #               {
  #                 template = "https://www.google.com/search";
  #                 params = [
  #                   {
  #                     name = "q";
  #                     value = "{searchTerms}";
  #                   }
  #                   {
  #                     name = "udm";
  #                     value = "14";
  #                   }
  #                 ];
  #               }
  #             ];
  #             definedAliases = ["@g"];
  #           };
  #         };
  #       };
  #       extensions = with inputs.firefox-addons.packages.${pkgs.system}; [
  #         ublock-origin
  #         bitwarden
  #         vimium
  #         tabliss
  #         privacy-badger
  #         reddit-enhancement-suite
  #         catppuccin
  #       ];
  #     };
  #   };
  # };
}
